#!/usr/bin/env python3
"""
This is one of two or three modules that contain function to
support brother print
"""
#  FOR COLORS:  pip3 install ansicolors



import textwrap
import subprocess as sp

from PIL import Image, ImageDraw, ImageFont

from fire import Fire
import os


import qrcode
from pyql700 import imrenderer, sticker

import numpy as np
# print("________________________________")

# print("""
# ./multextimg.py mk 640 9 'Žluťoučký kůň úpěl ďábelské ódy.'
# ./multextimg.py mk 640 20 'Žluťoučký
# kůň
# úpěl
# ďábelské
# ódy.'
# """)
# print("________________________________")


def print_tiv_file(pic):
    """
    simple tiv in python : imrenderer : file
    """
    imrenderer.render(pic)
    return


def print_tiv_image(img):
    """
    simple tiv in python : imrenderer : PIL img

    """
    # print("D... trying to render PIL image, convert to numpy", img)
    imrenderer.image_render( np.array(img) )
    return

    # rows, columns = os.popen('stty size', 'r').read().split()
    # tivsize = int(columns)
    # if tivsize <= 1:
    #     tivsize = 2
    # print("D... terminal", rows, columns, tivsize)

    # ok = False
    # try:
    #     res = sp.check_call("which tiv".split())
    #     ok = True
    # except: # pylint: disable=W0702
    #     ok = False
    # if not ok:
    #     print("D... tiv NOT present, do pip3 install tiv")
    #     # print("i... https://github.com/stefanhaustein/TerminalImageViewer")
    #     # pip3 install tiv
    #     return False
    # print("D... tiv present", res)
    # sp.check_call(("tiv -w "+str(tivsize)+" "+pic).split())
    return True


def wraptext(txt="ahoj", nchars=40):
    """
    returns multiline string, max char per line==n
    """
    #    return "\n".join(textwrap.fill(txt, width=n))
    # print("i... prewrap=", txt )
    byline = txt.split("\n")
    final = ""
    j = 0
    for i in byline:
        j += 1
        # print("D...LINE {:2d}/{:2d}==================".format(j,len(byline)) )
        res = "\n".join(textwrap.wrap(i, width=nchars))
        # print("DW... i=",res)
        final += res+"\n"
    # print("i... wrapfinal=",final)
    return final.rstrip()






def deduce_font_size(imgx=640, txt="Ahoj, jak se mas", cover=1.00,
                     fontname='/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf'):
    """
    Deduces best font size for ONE textline
    """
    # print("D... deduce font size:________________________________")
    # print("... ", len(txt) )
    if len(txt) == 0:
        txt = "AHOJ"
    # print("... ", len(txt) )
    # txt = "Hello World"
    # FONTNAME="/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf"
    fontsize = 1  # starting font size

    # portion of image width you want text width to be
    img_fraction = cover

    remains = 0

    font = ImageFont.truetype(fontname, fontsize)
    # while font.getsize(txt)[0] < img_fraction*img.size[0]:
    while font.getsize(txt)[0] < img_fraction*imgx:
        # calulate what remains blank
        # remains= img_fraction*img.size[0]-font.getsize(txt)[0]
        remains = img_fraction*imgx-font.getsize(txt)[0]
        # print("D... remains",remains,
        # "     font.getsize(txt)[0]=",font.getsize(txt)[0])
        # iterate until the text size is just larger than the criteria
        fontsize += 1
        font = ImageFont.truetype(fontname, fontsize)
        print(" ... ... fontsize:", fontsize, end="\r")
        if fontsize > 100:
            break

    # optionally de-increment to be sure it is less than criteria
    fontsize -= 1
    if fontsize>20: # f36 total -3 fantastic     f48 -3 in total OK
        fontsize -= 1
    if fontsize>40: # f83 -3 in total works
        fontsize -= 1
    font = ImageFont.truetype(fontname, fontsize)

    print('i... final font size', fontsize)
    # print( 'i... remains blank', remains) #GOOD INFO
    # dx=int(remains/2)+int(  (1-cover)*img.size[0]/2  ) # justified on center
    dx = int(remains/2)+int((1-cover)*imgx/2)# justified on center


    return fontsize, dx, font.getsize(txt)[1]




def deduce_remains_blank(fontsize, imgx=640, txt="Ahoj, jak se mas",
                         cover=1.00,
                         fontname='/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf'):
    """
    Checks borders for One line text
    """
    # portion of image width you want text width to be
    img_fraction = cover
    remains = 0
    font = ImageFont.truetype(fontname, fontsize)
    # calulate what remains blank
    # remains= img_fraction*img.size[0]-font.getsize(txt)[0]
    remains = img_fraction*imgx-font.getsize(txt)[0]
    # print( 'i... X=',img_fraction*imgx,
    #  "FontSize=",fontsize,"  TextWidth",font.getsize(txt)[0],
    #  'remains blank', remains)
    dx = int(remains/2)+int((1-cover)*imgx/2)# justified on center
    return dx




def deduce_height(text, smallest, fontname):
    """
    get text, deduce its height
    """
    print("D... deduce height:________________________________")
    # Determine text size using a scratch image.
    nlines = len(text.split("\n"))
    print("D... lines=", nlines)
    img = Image.new("RGBA", (1, 1))
    draw = ImageDraw.Draw(img)
    font = ImageFont.truetype(fontname, smallest)
    textsize = draw.textsize(text, font)
    return int(textsize[1]/nlines*1.06)*nlines





def deduce_fontsize2(width, text, wrapchar, cover=1.0,
                     limit=30):
    # ============= get fontsize =========================
    """
    for each textline looks for best font, checks blank-x
    returns fontsize,dx-start,ysizetot-useless
    """
    print("D... deduce font size 2:________________________________")
    smallest = -1
    dx = 0
    ysizetot = 0
    j = 0
    for i in wraptext(text, wrapchar).split("\n"):
        j += 1
        print("i... line {}/{}".format(j, len(wraptext(text, wrapchar).split("\n"))))
        fsize, dx, ysize = deduce_font_size(width, i, cover=cover)
        ysizetot += ysize
        if smallest == -1:
            smallest = fsize
        elif fsize < smallest:
            smallest = fsize

    if smallest > limit:
        smallest = limit
    print("D... recommended smallest fontsize={}".format(smallest),
          " ysize=", ysizetot)

    dxmin = -1
    for i in wraptext(text, wrapchar).split("\n"):
        # print("rem...",i)
        dx = deduce_remains_blank(smallest, width, i, cover=cover)
        if dxmin == -1:
            dxmin = dx
        elif dxmin > dx:
            dxmin = dx
    print("i... minimal found dx=", dxmin)
    return smallest, dxmin, ysizetot





#################################################
#           BEST THING HERE - CENTER OF ALL
#

def create_image_multiline(width, wrapchar, text, cover=1.0, xposition="center",
                           fontname='/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf',
                           limit=140):
    """
    Creates Image from the TEXT given inline, returns real image
    """
    #
    # version None to determine automat (1-40)
    # correction L(7%) H(30%)
    #
    qr = qrcode.QRCode(
        version=None,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=20,
        border=1,
    )
    qr.add_data(text)
    qr.make(fit=True)
    qrimg = qr.make_image()

    smallest, dx, ysizetot = deduce_fontsize2(width, text, wrapchar, cover=cover, limit=limit)

    #if smallest>10:
    #smallest=10 # really tiny
    #smallest=20 # possible to see well, but can be bigger
    #============ get height ======================
    # nlines=len( wraptext(text, wrapchar).split("\n") )
    # HEIGHT=ysizetot
    # print("i... height multiline         =", HEIGHT , "with",nlines,"lines")
    height = deduce_height(wraptext(text, wrapchar), smallest, fontname)
    # print("i... height from scratch image=", HEIght , "with",nlines,"lines")

    # ===============action===============================
    print("i... creating image", width, "x", height)
    img = Image.new('RGB', (width, height), color=(255, 255, 255))
    draw = ImageDraw.Draw(img)
    font = ImageFont.truetype(fontname, smallest)
    if xposition=="left": # prepare for qw position right
        dx=0
    draw.text((dx, 0), wraptext(text, wrapchar), font=font, fill="#000000")
    outfile = "z_output.png"
    qroutfile = "z_outputqr.png"

    outfile = "just_print707.jpg"
    qroutfile = "just_print707qr.jpg"
    print("")
    img.save(outfile)
    qrimg.save(qroutfile)
    print_tiv_file(qroutfile)
    print_tiv_file(outfile)
    return img, qrimg #
    return outfile



def qr(text, p=False):
    '''
qr sometext -p   ...  makes qrcode and prints it
    '''
    #
    # version None to determine automat (1-40)
    # correction L(7%) H(30%)
    #
    qroutfile = "z_outputqr.png"
    S62=62

    qr = qrcode.QRCode(
        version=None,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=20,
        border=1,
    )
    qr.add_data(text)
    qr.make(fit=True)
    qrimg = qr.make_image()

    qrimg.save(qroutfile)
    print_tiv_file(qroutfile)
    if p:
        sticker.print_image( S62, qrimg )
    return


if __name__ == "__main__":

    Fire({"mk":create_image_multiline})
    # quit()
    # wrapchar2 = 45 # Characters
    # width2 = 640
    # text2 = "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."

    # cover2 = 1 # full image width
    # fontname2 = '/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf'

    # create_image_multiline(width2, text2, wrapchar=55)



#!/usr/bin/env python3
# -fire CLI
from fire import Fire
from Pyql700.version import __version__

import subprocess as sp

import math
from PIL import Image, ImageDraw, ImageFont



from Pyql700.multextimg import create_image_multiline, print_tiv_image, print_tiv_file

import tempfile

import sys








print("i... module pyql700/stripes is being run")




#======================================================
#      CALLED FROM BINFILE
#
#def mult(S62, text, p=False):
# width I know it must be 707 and -10 +paste on 707
#def mult( S62, text, p=False, qr=False, qd=False):
def txt( S62, text, p=False, qr=False, image=None, qd=False, cover=0.7):
    """
    62 "some text\nmultiline" --qr (qr is autocreated)
    """
    # print("D... -p=", p, type(p) )
    if type(p)!=bool:
        print("X... some problem with -p")
        sys.exit(1)
    #print("D... ",S62,text,"print=",p,qr,qd)
    OUTNAME = "just_print707.jpg"

    qrminimy = 120 # minimal height for QR
    #S62=62
    if S62!=62:
        print("X... problem - only 62")
        sys.exit(1)
    # cover = 1.0
    xposition = "center"
    if (qr) or not (image is None):
        #---------- put text LEFT--------
        #cover = 0.7 # compress text
        xposition = "left"
        #text=text+"\n\n." # add empty line
    elif qd:
        text=text+"\n _" # add empty line
        text=text+"\n _" # add empty line
    else:
        cover = 1.0

    timg,qrimg = create_image_multiline(707, 40, text, cover=cover, xposition=xposition)
    print("D... IMAGE====",image)
    if not (image is None):
        #qrimg = img( S62, image , p=False)
        qrimg = Image.open(image)

    print("D... QRIMG LOADED, TIMG printed")
    print_tiv_image(timg)

    if (qr) or not (image is None):
        print("D... QRIMG LOADED, QRIMG/IMAGE printed")
        print_tiv_image(qrimg)

    if (qr) or not (image is None):
        print("D... attaching T right ", timg.size)
        print("D... attaching QR right", qrimg.size)

        if timg.size[1]<qrminimy: # I dont want smaller heigth than 120...
            print("D... TOO SMALL HEIGHT")
            nimage = Image.new('RGB', (timg.size[0], 120), (255,255,255))
            nimage.paste(timg,(0,0) )
            timg = nimage

        if timg.size[1]<qrimg.size[1]: # height problem, QR is higher
            print("D... QR/IMAGE have TOO BIG HEIGHT down with qrimg ... height",
                  timg.size,
                  qrimg.size
                  )
            #qrimg = qrimg.resize( (qrimg.size[0], img.size[1]) ) # placaty
            if qr:
                qrimg = qrimg.resize( (timg.size[1], timg.size[1]) )
            else:
                print("D... resizing down --image TO", ( int(qrimg.size[0]/qrimg.size[1]* timg.size[1]), timg.size[1]) )
                qrimg = qrimg.resize( ( int(qrimg.size[0]/qrimg.size[1]* timg.size[1]), timg.size[1]) )
                print("D... NEW qrimg --image  SIZE",qrimg.size)
            print("D... QRIMG CORRECT SIZE????")
            print_tiv_image(qrimg)


        if (1-cover)*timg.size[0] < qrimg.size[0]: # width problem, QR too wide
            allow_w = int( (1-cover)*timg.size[0] )
            print("D... qrimg TOO WIDE -down with qrimg ... allowed width=",
                  allow_w)
            if qr:
                qrimg.resize( (allow_w,allow_w ) )
            else:
                print("D... downscaling the WIDTH of qrimg --image alone",qrimg.size[1] )
                temp_h = qrimg.size[0]
                temp_w = qrimg.size[1]
                qrimg = qrimg.resize( (  allow_w , temp_w ) )
            # qrimg = qrimg.resize( (w,w ) )

            print("D... QRIMG CORRECT SIZE????", qrimg.size )
            print_tiv_image(qrimg)



        # img.paste(qrimg,( int(img.size[0]/2),0)) # paste right - stupid center
        timg.paste(qrimg,( timg.size[0] - qrimg.size[0] - 10,0)) # paste right
        print("D... QRIMG pasted on timg; final TIMG printed")
        print_tiv_image(timg)
        # img.save("z_finale.png")
        timg.save( OUTNAME )
    elif qd:
        print("D... attaching qr down")
    print_image( S62, timg , p)









def func():
    print("D... function defined in pyql700:stripes")
    return True

def test_func():
    print("D... test function ... run pytest")
    assert func()==True






#======================================================
#
# need to create 707 and paste 707-10
#
def print_file( media62 , jpgfile, p=False):
    """
    print JPG file with LPR. Core for the  printing. CREATE 707-10. Paste
    """
    print("D... printfile - open image")
    im = Image.open( jpgfile )
    print_image(media62, im, p)



def print_image( media62 , im, p=False):
    """
    print PIL Image. Anyway, saves to disk to LPR
    """
    prnname="just_print707.jpg" # temporaryname
    WIDTH, HEIGHT = im.size
    print("D... old size: w x h", WIDTH, HEIGHT)
    HEIGHT=int(HEIGHT*707/WIDTH)+10+10
    WIDTH=707
    print("D... new size w x h:", WIDTH, HEIGHT)
    imgnew = Image.new('RGB', (WIDTH, HEIGHT ), color = (255,255,255) )
    #imgnew.paste( im, (10,10)  ) #--------THIS I KNOW FROM TESTS
    # seems ok
    print("D... pasting old to new")
    imgnew.paste( im, (10,10)  ) #--------THIS I KNOW FROM TESTS
    print("D... SAVING", prnname )
    imgnew.save( prnname )

    # HERE I decide if to use CUPS or  pip3 brother_ql

    NAME=get_printer_name()

    if NAME is None:
        CMD = "brother_ql -b linux_kernel -p /dev/usb/lp0  -m QL-700 print -l 62 "+prnname
    else:

        CMD="lpr  -o page-left=3 -o page-right=3 -o page-top=0 -o page-bottom=0  -o media="+str(int(media62))+"X1  -o BrCutAtEnd=OFF  -o BrCutLable=0 -o BrTrimtape=ON -o BrPriority=BrQuality  -o  orientation-requested=3  -P "+NAME+" "+prnname


    print( "P...",CMD )
    if p:
        try:
            res=sp.check_output( CMD.split() ).decode("utf8")
            #res=False #sp.check_output( CMD.split() ).decode("utf8")
        except:
            res="!... Probably not installed. Printer name=/"+NAME+"/"
        print(res)
    return










#======================================================

def textline( amedia, text , p=False ):
    """
    say 62 text ; text is created  horizontally
    """
    print("i... amedia == 62" )
    media=amedia/10  # float  MM to  CM

    OUTPUT="z_output.png"
    OUTPUT="just_print707.jpg"
    CMD="convert -background white  -fill black  -font Gecko -pointsize 32          -size x120  label:'"+text+"' "+OUTPUT

    CMD="convert -background white  -fill black  -font Gecko -pointsize 32          -size x120".split()
    CMD.append("label:"+text)
    CMD.append(OUTPUT)
    print( "i...",CMD )
    res=sp.check_output( CMD ).decode("utf8")
    print(res)
    if p:
        print_file( amedia, OUTPUT )



def get_printer_name():
    """
    use lptstat to find the printer name QL700
    """
    CMD="lpstat -a"
    prnname=""
    print( "i...",CMD )
    ok = False
    try:
        res=sp.check_output( CMD.split() ).decode("utf8")
        ok = True
    # except CalledProcessError:
    except Exception as ex:
        print("!... exception - probably no printer installed\n",ex)
    if not ok:
        return None
    a=res.split("\n")
    for i in a:
        if i.find("QL")>=0:
            prnname=i.split()[0]
    print("i... Printer name==", prnname)
    return prnname

#======================================================

#def image( amedia, pic , p=False):
def img( amedia, pic , p=False):
    """
      62 picname.jpg  # picture is printed
    """
    if type(p)!=bool:
        print("X... some problem with -p")
        sys.exit(1)
    if amedia!=62:
        print("X... i know only 62")
        sys.exit(1)
    print("i... amedia == 62" )
    media=amedia/10  # float  MM to  CM
    #inputfile=args.inputpic

    inputfile=pic

    width=0
    width=int(300/2.56*media)
    #
    #===============
    #
    width=707-10
    #
    # this will make 10 border
    if media==6.2:
        print("6.2")
    elif media==2.9:
        print("2.9")
    else:
        print("x... not known media!", amedia)
        sys.exit(1)
    #========== IMAGE CREATE
    #OUTPUT="z_output.png"

    # ERROR ========= OUTPUT=pic
    OUTPUT = "/tmp/ql700_"+next(tempfile._get_candidate_names())+".png"

    #
    CMD="convert "+inputfile+" -auto-level  -scale "+str(width)+"x   -monochrome -dither FloydSteinberg  -remap pattern:gray50  "+OUTPUT


    CMD="-auto-level  -scale "+str(width)+"x   -monochrome -dither FloydSteinberg  -remap pattern:gray50  "+OUTPUT

    CMD = CMD.split()
    CMD.insert(0,inputfile)
    CMD.insert(0,'convert')

 #   CMD="convert "+inputfile+" -auto-level  -scale x"+str(width)+" -colorspace gray -ordered-dither o8x8 "+OUTPUT
 #   CMD="convert "+inputfile+" -auto-level  -scale x"+str(width)+" -colorspace gray -ordered-dither o8x8 "+OUTPUT
#    CMD="convert "+inputfile+" -auto-level  -scale x"+str(width)+" -colorspace gray -remap pattern:gray50 "+OUTPUT

    #
    print( "i...",CMD )
    res=sp.check_output( CMD ).decode("utf8")
    print("r...",res)
    #
    # TEXT
    #    CMD="convert -size x"+str(width)+" -background white  -pointsize 16 -font Courier -fill black -gravity NorthWest caption:"`cat PASKY.txt`"   -flatten  z_output.png"
    #

    print(OUTPUT)

    print_tiv_file(OUTPUT)
    print_file( amedia,  OUTPUT , p )
    return OUTPUT

#======================================================



#
# formerly I created a picture and then I reduced it and pasted
# to a new one.
#
# Now I want to set the tape width and 'variable' length
#
class Ql700Label(object):
    def __init__(self, tapewidth, tapelength=0 ):
        print("i... init")
        self.tapewidth= tapewidth # 62 (x29) OR 2.3 (x1.1),
        self.set_output()


#======================================================

def stripes( to=10, le=10, bo=10, ri=0, wi=707, p=False):
    """
    print/create stripes - test the borders and width
    """
    print("!... 62 can do width 707:300dpi (double is 1414 too much)....!")
    print("!... top,left,bottom,right = 10,10,10,0 .with  -o pl=3 pr=3...!")
    print("!... PIC SIZE= 697 x Height-20" )
    w, h = 640, 480
    w, h = 320,240
    w, h = 160,30
    left,top=20,10

    h=int(h/w*wi)
    fontsize = 12  # starting font size
    fontsize=int(fontsize/w*wi)
    #=================rewrite hard with parameters
    left=le
    top=to
    bottom=bo
    right=ri
    w=wi
    # creating new Image object
    img = Image.new("RGB", (w, h), (255,255,255))

    # create line image
    img1 = ImageDraw.Draw(img)
    FONTNAME='/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf'
    font = ImageFont.truetype(FONTNAME, fontsize)
    img1.text( (left,top),   "{}x{}+l{}+t{}-b{}-r{}".format(w,h,
                                                        le,top,
                                                        bottom,right) ,
               font=font, fill="#000000" )

    for i in range(left,w):
        if i%5==0:
            shape = [(i, top), (i , top+h)]
            img1.line(shape, fill ="black", width = 0)
    #img.show()

    #left top bottom right
    #rovna
    shape= [(le, int(top+h/2)  ), (w , int(top+h/2))]
    img1.line(shape, fill ="black", width = 0)
    #dolu
    shape= [(le, int(top+h/2)), (w , int(top+h-bottom))]
    img1.line(shape, fill ="black", width = 0)
    #nahoru
    shape= [(le, int(top+h/2)), (w , int(top))]
    img1.line(shape, fill ="black", width = 0)


    OUTFILE="out.jpg"
    img.save(OUTFILE)
    print("D... print:", OUTFILE)
    if p:
        print_file( 62, OUTFILE)
    else:
        img.show()


#======================================================


#======================================================



#======================================================


if __name__=="__main__":
    print("D... in main of project/module:  pyql700/stripes ")
    print("D... version :", __version__ )
    Fire( {'image':image,
           'file':print_file,
           'textline':textline,
           'stripes':stripes,
           'mult':mult,
           'name':get_printer_name
    })



#!/usr/bin/env python3

import subprocess
import sys
import subprocess as sp
import os
import pathlib

#================= import packages hard way=====================
import hashlib
import random
import unicodedata  #names

import glob

import_list=['fire', 'alive_progress','time']

#==========================================install check_call======
def install(package):
    subprocess.check_call([sys.executable, "-m", "pip", "install", package])

#==========================================install import======
def install_and_import(packagelist):
    import importlib
    for i in packagelist:
        try:
            importlib.import_module(i)
        except ImportError:
            install(i)
        finally:
            globals()[i] = importlib.import_module(i)

install_and_import( import_list )

#
#====================================== imports after install
#
from fire import Fire
from alive_progress import alive_bar,config_handler



#====================================================== RUN CODE
def launch(CMD, fake=False):
    #print("D... launchng", CMD)
    ok=False
    if not fake:
        try:
            res=sp.check_output( CMD.split() ).decode("utf8").rstrip()
            ok=True
        except:
            #print("D... CMD call ended !=0....")
            ok=False
        if not ok:
            return None
        #print("D...",res)
        return res



#======================================================main
def chkpass(passw="123456", sha_path="./"):
    if sha_path[-1]!="/":
        sha_path += "/"
    LIST=sha_path+'pwned-passwords-sha1-ordered-by-hash-v5.txt'
    if not os.path.exists(LIST):
        #print("D... LIST EXISTS")
    #else:
        print("D... LIST NOT EXIST !!!!!!!!!",LIST)
        quit()
    print("D... looking for password:", passw.strip() , end="")

    # encode the string
    encoded_str = passw.encode()

    # create a sha1 hash object initialized with the encoded string
    hash_obj = hashlib.sha1(encoded_str)

    # convert the hash object to a hexadecimal value
    hexa_value = hash_obj.hexdigest().upper()
    #print("HEX:",hexa_value)
    res=launch("sgrep "+hexa_value+" "+LIST)
    if not res is None:
        #print(res)
        count=res.split(":")[1]
        print("                    used {:5d}x: PAWNED!!". format(int(count)) ,end="")
        return count
    else:
        print("                                   ... ok", end="")
        return 0


#========================================== support to generatre keys
def strip_accents(text):
    try:
        text = unicode(text, 'utf-8')
    except NameError: # unicode is a default on python 3
        pass
    text = unicodedata.normalize('NFD', text)\
           .encode('ascii', 'ignore')\
           .decode("utf-8")
    return str(text)


#==================================================== replace things =========
def generate_keys(s):
    """
    I want to replace keys I want
    """
    names=["Jiří	  ","Jan	  ","Petr	  ",
           "Josef	  ","Pavel	  ","Jaroslav ",
           "Martin	  ","Miroslav ","Tomáš	  ",
           "František","Zdeněk	  ","Václav	  ",
           "Karel	  ","Milan	  ","Michal	  ",
           "Vladimír ","Lukáš	  ","David	  ",
           "Ladislav ","Jakub	  ","Stanislav",
           "Roman	  ","Ondřej	  ","Antonín  ",
           "Radek	  ","Marek	  ","Daniel	  ",
           "Miloslav ","Vojtěch  ",  "Jaromír  ",
           "Filip	  ","Ivan	  ","Aleš	  ",
           "Libor	  ","Oldřich  ",  "Rudolf	  ",
           "Vlastimil","Jindřich ","Miloš	  ",
           "Adam	  ","Lubomír  "]
    names=[ strip_accents(i.strip()) for i in names]
    for i in names:
        print(i, end=",")
    print()


#=================================== SUPPORT FUNC
def get_pass_codes():
    files=glob.glob("*.subst")
    for fil in files:
        print("D... opening",fil)
        with open(fil) as f:
            res=f.readlines()
        #print(res)
        passrepl={}
        for i in res:
            #print(i)
            key=i.split()[0]
            value=i.split()[1].split(",")
            passrepl[key]=value
        #print(passrepl)
        return passrepl
    print("!... no .subst file")
    quit()

#============================================ ONE FUNC
def subst_pass(*page_file, sha_path="./", printit=False):  #checkonly=True
    """
    I can check the passwords in .page file; or print it
    """
    if isinstance(page_file, list) and  len(page_file) != 0:
        res=[]
        for i in page_file:
            print("D...subst_pass: PAGE--------------------",i)
            res.append(subst_pass(i, sha_path=sha_path, printit=printit))
        print("D... returning:", res, len(res))
        return res
    print("D... on track subst_pass:", page_file)
    fname=page_file[0]
    passrepl=get_pass_codes()
    with open(fname) as f:
        page = f.readlines()
    outtxt=""
    # ==================== replace on each line
    for line in page:
        #print("D... checking line:", line.rstrip() )
        newline = ""
        #================ check p:
        if line.find("p:") >= 0:
            pwd=line.split("p:")[-1]
            pwd=pwd.split(" ")[0]
            chkpass(pwd.rstrip(), sha_path) # I need to strip
        for k in passrepl.keys():
            #print("KEY...",k)

            #================ look for key presence:
            line2=line.lower()
            if line2.find(k) >= 0:
                pos = line2.find(k)
                lent = len(k)
                true_key = line[pos:pos+lent]
                true_map = [ c.isupper() for c in  true_key]
                i = random.randrange(len(passrepl[k]))
                false_key = passrepl[k][i]
                false_key2 = ""
                if len(false_key) != len(true_map):
                    print("D...  PROBLEM !!!!!!!!!!!!!!!!!!!", true_map, false_key)
                    quit()
                for i in range(len(false_key)):
                    if true_map[i]:
                        false_key2 += false_key[i].upper()
                    else:
                        false_key2 += false_key[i].lower()
                #print("D...   KEY FOUND at",pos," :", line.strip())
                line2 = line2[:pos] + false_key2 + line2[pos+lent:]
                if printit:
                    print("\nD...   newline     ",pos," :", line2.strip())
                #outtxt += line2
                if newline == "":
                    newline = line2
        print()
        if newline == "":
            newline = line
        outtxt += newline
    #print("D... output:______________________________________________\n")
    #print(outtxt)
    if not printit:
        return ""
    return outtxt



#========================================================MAIN======
#========================================================MAIN======
#========================================================MAIN======
if __name__ == "__main__":
    #Fire(replace_keys )
    Fire( subst_pass )



#!/usr/bin/env python3

#  FOR COLORS:  pip3 install ansicolors

# https://raw.githubusercontent.com/djentleman/imgrender/master/imgrender/main.py
print("D... pip3 install ansicolors")
from colors import color
from PIL import Image
import numpy as np
import argparse

from fire import Fire
import os


class Renderer():

    def get_pixel(self, col):
        return color('  ', bg=f'rgb({int(col[0])}, {int(col[1])}, {int(col[2])})')

    def render_image(self, pixels, scale):
        # first of all scale the image to the scale 'tuple'

        # JM
        image_size = pixels.shape[:2]

        if isinstance(scale,int):
            scale= ( int(scale/image_size[1]*image_size[0]),scale)

        print("D...    imagesize ", image_size)
        print("D...    imagesize2", scale)
        rows, columns = os.popen('stty size', 'r').read().split()

        #  adapt to imrenderer....
        rows, columns = int(rows)-1, int((int(columns)-1)/2) # I MUST DIVIDE TERMINAL WIDTH BY 2; TIV was OK /1
        tivsize = columns
        if tivsize <= 1:
            tivsize = 2
        print("D...    terminal", rows, columns, " tiv -w",tivsize)


        if (scale[0]>rows) or (scale[1]>columns) or (scale[0]==0) or (scale[1]==0):

            scale1= ( int(columns/image_size[1]*image_size[0]), columns)

            scale2= ( rows, int(rows/image_size[0]*image_size[1]))
            print("D...    compeeting sizes:",scale1,scale2)
            if scale1[0]>scale2[0]:
                scale=scale2
            else:
                scale=scale1

            #scale=(rows,columns)

        print("D...    imagesize3", scale)

        block_size = (image_size[0]/scale[0], image_size[1]/scale[1])
        blocks = []
        y = 0
        while y < image_size[0]:
            x = 0
            block_col = []
            while x < image_size[1]:
                # get a block, reshape in into an Nx3 matrix and then get average of each column
                block_col.append(pixels[int(y):int(y+block_size[0]), int(x):int(x+block_size[1])].reshape(-1, 3).mean(axis=0))
                x += block_size[1]
            blocks.append(block_col)
            y += block_size[0]
        output = [[self.get_pixel(block) for block in row] for row in blocks]
        return output



def img2numpy(img):
    print("D...    img.shape", img.shape)

    if len(img.shape)==2:
        print("D...    special GRAY/B+W image", type(img[0,0]))
        if isinstance(img[0,0], np.bool_):
            print("D...    Boolean colors: B+W image ")
            npar = np.array([[ [255,255,255] if pixel else [0,0,0] for pixel in row] for row in img])
            #print(npar)
            return npar
        #print(img)
        #npar = np.array([[pixel for pixel in row] for row in img])
        #print(npar)
        return np.array([[ [pixel,pixel,pixel] for pixel in row] for row in img])
    if img.shape[2] > 3:
        return np.array([[pixel[:3] for pixel in row] for row in img])
    return img


def get_image(path):
    img = np.asarray(Image.open(path))
    return img2numpy(img)


def image_render(image, scale=(0, 0)):
    """
    takes image
    """
    renderer = Renderer()
    #image = get_image(path)
    image = img2numpy(image)

    output = renderer.render_image(image, scale)
    print('\n'.join([''.join(row) for row in output]))



def render(path, scale=(0, 0)):
    """
    ORIGINAL PART FROM FILE
    """
    renderer = Renderer()
    image = get_image(path)

    output = renderer.render_image(image, scale)
    print('\n'.join([''.join(row) for row in output]))

if __name__=="__main__":
    Fire()



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

#
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

import os.path

def readver(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in readver(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")




setup(
    name="pyql700",
    description="Automatically created environment for python package",
    url="https://gitlab.com/jaromrax/pyql700",
    author=" me ",
    author_email="me@example.com",
    licence="GPL2",
    version=get_version("Pyql700/version.py"),
    packages=['Pyql700'],
    package_data={'pyql700': ['data/*']},
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    scripts = ['bin/pyql700'],
    install_requires = ['numpy',"ansicolors","qrcode","termcolor"],
)
